/*
 * hmi.cpp
 *
 *  Created on: 22.02.2015
 *      Author: wn
 */

#include <msp430g2553.h>
#include <isr_compat.h>
#include <stdint.h>

#include "hmi.h"
#include "pwm.h"
#include "mySpi.h"
#include "display.h"


class SwitchPort2 {
public:
	SwitchPort2(uint16_t bit) : m_bit(bit), m_state(0), m_cnt(0) {
		P2REN |= m_bit;
	};
	bool get() {
		bool result = false;
		switch (m_state) {
		case 0:
			if (P2IN & m_bit) {
				m_cnt = 0;
			} else {
				m_cnt++;
			}
			if (m_cnt > 100) {
				m_state = 1;
				m_cnt = 0;
			}
			break;
		case 1:
			result = true;
			m_state = 2;
			break;
		case 2:
			if (P2IN & m_bit) {
				m_cnt++;
			} else {
				m_cnt = 0;
			}
			if (m_cnt > 20) {
				m_state = 0;
				m_cnt = 0;
			}
			break;
		default:
			m_state = 0;
			break;
		}
		return result;
	};
private:
	uint16_t m_bit;
	uint8_t m_state;
	uint16_t m_cnt;
};


SwitchPort2 switch1 = SwitchPort2(BIT0);
SwitchPort2 switch2 = SwitchPort2(BIT1);
SwitchPort2 switch3 = SwitchPort2(BIT2);

void hmiInit() {
	spiInit();
	dispInit();
}

void hmiExec() {
	static float savedVoltage = 0.0;
	static bool enabled = false;
	if (switch1.get()) {
		if (enabled) {
			enabled = false;
			setUDes(0);
		} else {
			enabled = true;
			setUDes(savedVoltage);
		}
	}
	if (enabled && switch2.get()) {
		savedVoltage = getUDes() - 1.0;
		setUDes(savedVoltage);
	}
	if (enabled && switch3.get()) {
		savedVoltage = getUDes() + 1.0;
		setUDes(savedVoltage);
	}

	static uint32_t cnt = 0;
	cnt++;
	if (cnt >= 10000) {
		dispSetFloat(0, getUCur());
		dispSetFloat(1, getUDes());
		cnt = 0;
	}
}
